function scrape(){
    function getVisibleLinkedPosts()
    {
        const link_elems = document.getElementsByTagName("a");
        let matching_links = []

        for(let i = 0; i != link_elems.length; ++i){
            const link_href = link_elems[i].href;
            if(link_href.match(/.*\/p\/.*\//)){
                matching_links.push(link_href)
            }
        }   
        return matching_links
    }

    function scrollHalfAPage()
    {
        const initialScrollY = window.scrollY;
        const initialScrollX = window.scrollX;
        const windowHeight = window.innerHeight;

        window.scrollBy(0, windowHeight / 2);

        return window.scrollY - initialScrollY > 10;
    }

    function mergeArrays(array1, array2)
    {
        const filtered2 = array2.filter( elem2 => !array1.some(elem1  => elem2 === elem1))
        return array1.concat(filtered2)
    }

    function getPageName(){
        const path = window.location.pathname
        const pathItems = path.split("/")
        let page = pathItems.pop()
        while(page == ""){
            page = pathItems.pop()
        }
        return page
    }

    function downloadAsCsv(results)
    {
        const csvString = results.map( pageLink => pageLink).join("\n")
        const blob = new Blob([csvString], {type:"text/csv"})
        const blobUrl = URL.createObjectURL(blob);
        const downloadLink = document.createElement('a')
        downloadLink.href = blobUrl
        downloadLink.download = `insta_links_${getPageName()}.csv`
        downloadLink.click()
    }

    function save_results(results)
    {
        instaLinksScrollPluginData.currentlyActiveScrape = null
        console.log("===============")
        console.log(" Done Scraping ")
        console.log("===============")
        console.log(`Found ${results.length} posts linked`)
        downloadAsCsv(results)
        console.log("===============")
    }

    const scrapeStep = (scrapedBefore) => () => {
        console.log("Scraping...")
        const newScraping = getVisibleLinkedPosts()
        const scrapedNow = mergeArrays(scrapedBefore, newScraping);
        console.log(`Scraped ${scrapedNow.length} links so far`)
        if(scrollHalfAPage())
        {
            instaLinksScrollPluginData.currentlyActiveScrape = setTimeout(scrapeStep(scrapedNow), 1500)
        }
        else
        {
            console.log("Done.")
            instaLinksScrollPluginData.currentlyActiveScrape = null
            save_results(scrapedNow)
        }
    } 
    if(instaLinksScrollPluginData.currentlyActiveScrape === null)
    {
        console.log("==================")
        console.log("  Scrape Started  ")
        console.log("==================")
        window.scrollTo(0,0)
        scrapeStep([])()
    }
}

scrape()